package frontend

import (
	"html/template"
	"log"
	"net/http"
	"path/filepath"

	b "streifling.com/jason/cpolis/cmd/backend"
)

func CreateTag(c *b.Config, s map[string]*Session) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		if _, err := ManageSession(w, r, c, s); err != nil {
			http.Error(w, "Die Session ist abgelaufen. Bitte erneut anmelden.", http.StatusUnauthorized)
			return
		}

		tmpl, err := template.ParseFiles(filepath.Join(c.WebDir, "templates", "add-tag.html"))
		if err = template.Must(tmpl, err).ExecuteTemplate(w, "page-content", nil); err != nil {
			log.Println(err)
			http.Error(w, err.Error(), http.StatusInternalServerError)
			return
		}
	}
}

func AddTag(c *b.Config, db *b.DB, s map[string]*Session) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		session, err := ManageSession(w, r, c, s)
		if err != nil {
			http.Error(w, "Die Session ist abgelaufen. Bitte erneut anmelden.", http.StatusUnauthorized)
			return
		}

		tag := r.PostFormValue("tag")
		if len(tag) == 0 {
			http.Error(w, "Bitte einen Tag eingeben.", http.StatusBadRequest)
			return
		}
		db.AddTag(tag)

		data := new(struct{ Role int })
		data.Role = session.User.Role

		tmpl, err := template.ParseFiles(filepath.Join(c.WebDir, "templates", "hub.html"))
		tmpl = template.Must(tmpl, err)
		if err = tmpl.ExecuteTemplate(w, "page-content", data); err != nil {
			log.Println(err)
			http.Error(w, err.Error(), http.StatusInternalServerError)
			return
		}
	}
}
